@@ -1,14 +1,13 @@
 import sys
-import weakref
 
 import kubernetes
 from dagster import EventMetadataEntry, Field, Noneable, StringSource, check
 from dagster.cli.api import ExecuteRunArgs
 from dagster.core.events import EngineEventData
 from dagster.core.host_representation import ExternalPipeline
-from dagster.core.instance import DagsterInstance
 from dagster.core.launcher import RunLauncher
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
+from dagster.core.storage.tags import DOCKER_IMAGE_TAG
 from dagster.serdes import ConfigurableClass, ConfigurableClassData, serialize_dagster_namedtuple
 from dagster.utils import frozentags, merge_dicts
 from dagster.utils.error import serializable_error_info_from_exc_info
@@ -55,7 +54,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         instance_config_map (str): The ``name`` of an existing Volume to mount into the pod in
             order to provide a ConfigMap for the Dagster instance. This Volume should contain a
             ``dagster.yaml`` with appropriate values for run storage, event log storage, etc.
-        postgres_password_secret (str): The name of the Kubernetes Secret where the postgres
+        postgres_password_secret (Optional[str]): The name of the Kubernetes Secret where the postgres
             password can be retrieved. Will be mounted and supplied as an environment variable to
             the Job Pod.
         dagster_home (str): The location of DAGSTER_HOME in the Job container; this is where the
@@ -94,8 +93,8 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         self,
         service_account_name,
         instance_config_map,
-        postgres_password_secret,
-        dagster_home,
+        postgres_password_secret=None,
+        dagster_home=None,
         job_image=None,
         image_pull_policy="Always",
         image_pull_secrets=None,
@@ -124,23 +123,24 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         self._batch_api = k8s_client_batch_api or kubernetes.client.BatchV1Api()
         self._core_api = k8s_client_core_api or kubernetes.client.CoreV1Api()
 
-        self.job_config = None
+        self._job_config = None
         self._job_image = check.opt_str_param(job_image, "job_image")
-        self._dagster_home = check.str_param(dagster_home, "dagster_home")
+        self.dagster_home = check.str_param(dagster_home, "dagster_home")
         self._image_pull_policy = check.str_param(image_pull_policy, "image_pull_policy")
         self._image_pull_secrets = check.opt_list_param(
             image_pull_secrets, "image_pull_secrets", of_type=dict
         )
         self._service_account_name = check.str_param(service_account_name, "service_account_name")
-        self._instance_config_map = check.str_param(instance_config_map, "instance_config_map")
-        self._postgres_password_secret = check.str_param(
+        self.instance_config_map = check.str_param(instance_config_map, "instance_config_map")
+        self.postgres_password_secret = check.opt_str_param(
             postgres_password_secret, "postgres_password_secret"
         )
         self._env_config_maps = check.opt_list_param(
             env_config_maps, "env_config_maps", of_type=str
         )
         self._env_secrets = check.opt_list_param(env_secrets, "env_secrets", of_type=str)
-        self._instance_ref = None
+
+        super().__init__()
 
     @classmethod
     def config_type(cls):
@@ -164,17 +164,13 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
     def inst_data(self):
         return self._inst_data
 
-    @property
-    def _instance(self):
-        return self._instance_ref() if self._instance_ref else None
-
-    def _get_static_job_config(self):
-        if self.job_config:
-            return self.job_config
+    def get_static_job_config(self):
+        if self._job_config:
+            return self._job_config
         else:
-            self.job_config = DagsterK8sJobConfig(
+            self._job_config = DagsterK8sJobConfig(
                 job_image=check.str_param(self._job_image, "job_image"),
-                dagster_home=check.str_param(self._dagster_home, "dagster_home"),
+                dagster_home=check.str_param(self.dagster_home, "dagster_home"),
                 image_pull_policy=check.str_param(self._image_pull_policy, "image_pull_policy"),
                 image_pull_secrets=check.opt_list_param(
                     self._image_pull_secrets, "image_pull_secrets", of_type=dict
@@ -183,22 +179,22 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
                     self._service_account_name, "service_account_name"
                 ),
                 instance_config_map=check.str_param(
-                    self._instance_config_map, "instance_config_map"
+                    self.instance_config_map, "instance_config_map"
                 ),
-                postgres_password_secret=check.str_param(
-                    self._postgres_password_secret, "postgres_password_secret"
+                postgres_password_secret=check.opt_str_param(
+                    self.postgres_password_secret, "postgres_password_secret"
                 ),
                 env_config_maps=check.opt_list_param(
                     self._env_config_maps, "env_config_maps", of_type=str
                 ),
                 env_secrets=check.opt_list_param(self._env_secrets, "env_secrets", of_type=str),
             )
-            return self.job_config
+            return self._job_config
 
     def _get_grpc_job_config(self, job_image):
         return DagsterK8sJobConfig(
             job_image=check.str_param(job_image, "job_image"),
-            dagster_home=check.str_param(self._dagster_home, "dagster_home"),
+            dagster_home=check.str_param(self.dagster_home, "dagster_home"),
             image_pull_policy=check.str_param(self._image_pull_policy, "image_pull_policy"),
             image_pull_secrets=check.opt_list_param(
                 self._image_pull_secrets, "image_pull_secrets", of_type=dict
@@ -206,9 +202,9 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             service_account_name=check.str_param(
                 self._service_account_name, "service_account_name"
             ),
-            instance_config_map=check.str_param(self._instance_config_map, "instance_config_map"),
-            postgres_password_secret=check.str_param(
-                self._postgres_password_secret, "postgres_password_secret"
+            instance_config_map=check.str_param(self.instance_config_map, "instance_config_map"),
+            postgres_password_secret=check.opt_str_param(
+                self.postgres_password_secret, "postgres_password_secret"
             ),
             env_config_maps=check.opt_list_param(
                 self._env_config_maps, "env_config_maps", of_type=str
@@ -216,12 +212,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             env_secrets=check.opt_list_param(self._env_secrets, "env_secrets", of_type=str),
         )
 
-    def initialize(self, instance):
-        check.inst_param(instance, "instance", DagsterInstance)
-        # Store a weakref to avoid a circular reference / enable GC
-        self._instance_ref = weakref.ref(instance)
-
-    def launch_run(self, instance, run, external_pipeline):
+    def launch_run(self, run, external_pipeline):
         check.inst_param(run, "run", PipelineRun)
         check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
 
@@ -236,7 +227,12 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         job_config = (
             self._get_grpc_job_config(repository_origin.container_image)
             if repository_origin.container_image
-            else self._get_static_job_config()
+            else self.get_static_job_config()
+        )
+
+        self._instance.add_run_tags(
+            run.run_id,
+            {DOCKER_IMAGE_TAG: job_config.job_image},
         )
 
         input_json = serialize_dagster_namedtuple(
@@ -256,9 +252,22 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             user_defined_k8s_config=user_defined_k8s_config,
         )
 
+        self._instance.report_engine_event(
+            "Creating Kubernetes run worker job",
+            run,
+            EngineEventData(
+                [
+                    EventMetadataEntry.text(job_name, "Kubernetes Job name"),
+                    EventMetadataEntry.text(self.job_namespace, "Kubernetes Namespace"),
+                    EventMetadataEntry.text(run.run_id, "Run ID"),
+                ]
+            ),
+            cls=self.__class__,
+        )
+
         self._batch_api.create_namespaced_job(body=job, namespace=self.job_namespace)
         self._instance.report_engine_event(
-            "Kubernetes run worker job launched",
+            "Kubernetes run worker job created",
             run,
             EngineEventData(
                 [
